@@ -2,10 +2,9 @@ import re
 from django.conf import settings
 from django.utils.html import escape
 from django.utils.encoding import smart_str
-from urlobject import URLObject, decode_query, URL_COMPONENTS
+from urlobject import URLObject, decode_query
 from django.template import StringOrigin, Lexer, Parser
 from django.template.defaulttags import kwarg_re
-from django.utils.datastructures import MultiValueDict
 from django.template import Template, Library, Node, TemplateSyntaxError
 
 
@@ -66,6 +65,10 @@ class SpurlURLBuilder(object):
         query = self.prepare_value(value)
         self.url = self.url.with_query(query)
 
+    def handle_query_from(self, value):
+        url = URLObject.parse(value)
+        self.url = self.url.with_query(url.query)
+
     def handle_add_query(self, value):
         query_to_add = self.prepare_value(value)
         if isinstance(query_to_add, basestring):
@@ -73,6 +76,12 @@ class SpurlURLBuilder(object):
         for key, value in query_to_add.items():
             self.url = self.url.add_query_param(key, value)
 
+    def handle_add_query_from(self, value):
+        url = URLObject.parse(value)
+        query_to_add = dict(decode_query(url.query))
+        for key, value in query_to_add.items():
+            self.url = self.url.add_query_param(key, value)
+
     def handle_set_query(self, value):
         query_to_set = self.prepare_value(value)
         if isinstance(query_to_set, basestring):
@@ -80,31 +89,78 @@ class SpurlURLBuilder(object):
         for key, value in query_to_set.items():
             self.url = self.url.set_query_param(key, value)
 
+    def handle_set_query_from(self, value):
+        url = URLObject.parse(value)
+        query_to_set = dict(decode_query(url.query))
+        for key, value in query_to_set.items():
+            self.url = self.url.set_query_param(key, value)
+
     def handle_remove_query_param(self, value):
         self.url = self.url.without_query_param(value)
 
+    def handle_toggle_query(self, value):
+        query_to_toggle = self.prepare_value(value)
+        if isinstance(query_to_toggle, basestring):
+            query_to_toggle = dict(decode_query(query_to_toggle))
+        current_query = self.url.query_dict()
+        for key, value in query_to_toggle.items():
+            if isinstance(value, basestring):
+                value = value.split(',')
+            first, second = value
+            if key in current_query and first in current_query[key]:
+                self.url = self.url.set_query_param(key, second)
+            else:
+                self.url = self.url.set_query_param(key, first)
+
     def handle_scheme(self, value):
         self.url = self.url.with_scheme(value)
 
+    def handle_scheme_from(self, value):
+        url = URLObject.parse(value)
+        self.url = self.url.with_scheme(url.scheme)
+
     def handle_host(self, value):
         host = self.prepare_value(value)
         self.url = self.url.with_host(host)
 
+    def handle_host_from(self, value):
+        url = URLObject.parse(value)
+        self.url = self.url.with_host(url.host)
+
     def handle_path(self, value):
         path = self.prepare_value(value)
         self.url = self.url.with_path(path)
 
+    def handle_path_from(self, value):
+        url = URLObject.parse(value)
+        self.url = self.url.with_path(url.path)
+
     def handle_add_path(self, value):
         path_to_add = self.prepare_value(value)
         self.url = self.url.add_path_component(path_to_add)
 
+    def handle_add_path_from(self, value):
+        url = URLObject.parse(value)
+        path_to_add = url.path
+        if path_to_add.startswith('/'):
+            path_to_add = path_to_add[1:]
+        self.url = self.url.add_path_component(path_to_add)
+
     def handle_fragment(self, value):
         fragment = self.prepare_value(value)
         self.url = self.url.with_fragment(fragment)
 
+    def handle_fragment_from(self, value):
+        url = URLObject.parse(value)
+        self.url = self.url.with_fragment(url.fragment)
+
     def handle_port(self, value):
         self.url = self.url.with_port(value)
 
+    def handle_port_from(self, value):
+        url = URLObject.parse(value)
+        self.url = self.url.with_port(url.port)
+
     def handle_autoescape(self, value):
         self.autoescape = convert_to_boolean(value)
 
@@ -120,12 +176,6 @@ class SpurlURLBuilder(object):
             value = self.render_template(value)
         return value
 
-    def convert_to_boolean(self, string_or_boolean):
-        if isinstance(string_or_boolean, bool):
-            return string_or_boolean
-        if isinstance(string_or_boolean, basestring):
-            return bool(TRUE_RE.match(string_or_boolean))
-
     def unescape_tags(self, template_string):
         """Spurl allows the use of templatetags inside templatetags, if
         the inner templatetags are escaped - {\% and %\}"""
